/*
 * I am the optimal position determination engine.
 *
 * Macro parameters:
 *   P  PV prefix (e.g. 17idb:)
 *   D  Mini-beam apparatus device name (e.g. Mb1)
 */

program minibeam
option +r;


%%#include <string.h>

%%#include <errlog.h>

%%#include "minibeamLib.h"


#define PROGRAM_NAME "minibeam"

#define SCAN_MPTS 500 /* Must agree with minibeam-auto-tune-axis.db */

#define SNCSEQ_PV_STAT_OK 0 /* Must match pvStatOK in include/pvAlarm.h */

#define COMPUTE_OPT_YES 1
#define COMPUTE_OPT_NO  0

#define COMPUTE_OPT_RETURN_OK    0
#define COMPUTE_OPT_RETURN_ERROR 1


short ApOpdm[8];
  assign ApOpdm[0] to "{P}{D}:Ap1Opdm";
  assign ApOpdm[1] to "{P}{D}:Ap2Opdm";
  assign ApOpdm[2] to "{P}{D}:Ap3Opdm";
  assign ApOpdm[3] to "{P}{D}:Ap4Opdm";
  assign ApOpdm[4] to "{P}{D}:Ap1Opdm";
  assign ApOpdm[5] to "{P}{D}:Ap2Opdm";
  assign ApOpdm[6] to "{P}{D}:Ap3Opdm";
  assign ApOpdm[7] to "{P}{D}:Ap4Opdm";

short ApComputeOpt[8];
  assign ApComputeOpt[0] to "{P}{D}:Ap1ComputeOptH";
  assign ApComputeOpt[1] to "{P}{D}:Ap2ComputeOptH";
  assign ApComputeOpt[2] to "{P}{D}:Ap3ComputeOptH";
  assign ApComputeOpt[3] to "{P}{D}:Ap4ComputeOptH";
  assign ApComputeOpt[4] to "{P}{D}:Ap1ComputeOptV";
  assign ApComputeOpt[5] to "{P}{D}:Ap2ComputeOptV";
  assign ApComputeOpt[6] to "{P}{D}:Ap3ComputeOptV";
  assign ApComputeOpt[7] to "{P}{D}:Ap4ComputeOptV";
  monitor ApComputeOpt;

short ApComputeOptReturn[8];
  assign ApComputeOptReturn[0] to "{P}{D}:Ap1CompOptRetrnH";
  assign ApComputeOptReturn[1] to "{P}{D}:Ap2CompOptRetrnH";
  assign ApComputeOptReturn[2] to "{P}{D}:Ap3CompOptRetrnH";
  assign ApComputeOptReturn[3] to "{P}{D}:Ap4CompOptRetrnH";
  assign ApComputeOptReturn[4] to "{P}{D}:Ap1CompOptRetrnV";
  assign ApComputeOptReturn[5] to "{P}{D}:Ap2CompOptRetrnV";
  assign ApComputeOptReturn[6] to "{P}{D}:Ap3CompOptRetrnV";
  assign ApComputeOptReturn[7] to "{P}{D}:Ap4CompOptRetrnV";

double ApSugPreset[8];
  assign ApSugPreset[0] to "{P}{D}:Ap1SugPresetH";
  assign ApSugPreset[1] to "{P}{D}:Ap2SugPresetH";
  assign ApSugPreset[2] to "{P}{D}:Ap3SugPresetH";
  assign ApSugPreset[3] to "{P}{D}:Ap4SugPresetH";
  assign ApSugPreset[4] to "{P}{D}:Ap1SugPresetV";
  assign ApSugPreset[5] to "{P}{D}:Ap2SugPresetV";
  assign ApSugPreset[6] to "{P}{D}:Ap3SugPresetV";
  assign ApSugPreset[7] to "{P}{D}:Ap4SugPresetV";

double ApScanPosData[8][SCAN_MPTS];
  assign ApScanPosData[0] to "{P}{D}:Ap1ScanH.P1RA";
  assign ApScanPosData[1] to "{P}{D}:Ap2ScanH.P1RA";
  assign ApScanPosData[2] to "{P}{D}:Ap3ScanH.P1RA";
  assign ApScanPosData[3] to "{P}{D}:Ap4ScanH.P1RA";
  assign ApScanPosData[4] to "{P}{D}:Ap1ScanV.P1RA";
  assign ApScanPosData[5] to "{P}{D}:Ap2ScanV.P1RA";
  assign ApScanPosData[6] to "{P}{D}:Ap3ScanV.P1RA";
  assign ApScanPosData[7] to "{P}{D}:Ap4ScanV.P1RA";

float ApScanDetData[8][SCAN_MPTS];
  assign ApScanDetData[0] to "{P}{D}:Ap1ScanH.D01DA";
  assign ApScanDetData[1] to "{P}{D}:Ap2ScanH.D01DA";
  assign ApScanDetData[2] to "{P}{D}:Ap3ScanH.D01DA";
  assign ApScanDetData[3] to "{P}{D}:Ap4ScanH.D01DA";
  assign ApScanDetData[4] to "{P}{D}:Ap1ScanV.D01DA";
  assign ApScanDetData[5] to "{P}{D}:Ap2ScanV.D01DA";
  assign ApScanDetData[6] to "{P}{D}:Ap3ScanV.D01DA";
  assign ApScanDetData[7] to "{P}{D}:Ap4ScanV.D01DA";

int ApScanNumPts[8];
  assign ApScanNumPts[0] to "{P}{D}:Ap1ScanH.NPTS";
  assign ApScanNumPts[1] to "{P}{D}:Ap2ScanH.NPTS";
  assign ApScanNumPts[2] to "{P}{D}:Ap3ScanH.NPTS";
  assign ApScanNumPts[3] to "{P}{D}:Ap4ScanH.NPTS";
  assign ApScanNumPts[4] to "{P}{D}:Ap1ScanV.NPTS";
  assign ApScanNumPts[5] to "{P}{D}:Ap2ScanV.NPTS";
  assign ApScanNumPts[6] to "{P}{D}:Ap3ScanV.NPTS";
  assign ApScanNumPts[7] to "{P}{D}:Ap4ScanV.NPTS";


int Status;
int Status2;
int Status3;
int Status4;
int Index;
long Count;
double OptimalPosition;

char *MacroP;
char *MacroD;


ss Main {
  state initialize {
    when () {
      MacroP = macValueGet("P");
      if (!MacroP) MacroP = "";
      MacroD = macValueGet("D");
      if (!MacroD) MacroD = "";
    } state waitForComputationNeeded
  }

  state waitForComputationNeeded {
    when (ApComputeOpt[0] == COMPUTE_OPT_YES) { Index = 0;
    } state computeOptimalPosition
    when (ApComputeOpt[1] == COMPUTE_OPT_YES) { Index = 1;
    } state computeOptimalPosition
    when (ApComputeOpt[2] == COMPUTE_OPT_YES) { Index = 2;
    } state computeOptimalPosition
    when (ApComputeOpt[3] == COMPUTE_OPT_YES) { Index = 3;
    } state computeOptimalPosition
    when (ApComputeOpt[4] == COMPUTE_OPT_YES) { Index = 4;
    } state computeOptimalPosition
    when (ApComputeOpt[5] == COMPUTE_OPT_YES) { Index = 5;
    } state computeOptimalPosition
    when (ApComputeOpt[6] == COMPUTE_OPT_YES) { Index = 6;
    } state computeOptimalPosition
    when (ApComputeOpt[7] == COMPUTE_OPT_YES) { Index = 7;
    } state computeOptimalPosition
  }

  state computeOptimalPosition {
    when () {
      ApComputeOptReturn[Index] = COMPUTE_OPT_RETURN_ERROR;
      Status  = pvGet(ApScanPosData[Index]);
      Status2 = pvGet(ApScanDetData[Index]);
      Status3 = pvGet(ApScanNumPts[Index]);
      Status4 = pvGet(ApOpdm[Index]);
      if (Status  == SNCSEQ_PV_STAT_OK &&
          Status2 == SNCSEQ_PV_STAT_OK &&
          Status3 == SNCSEQ_PV_STAT_OK &&
          Status4 == SNCSEQ_PV_STAT_OK) {
        Count = pvCount(ApScanDetData[Index]);
        if (ApScanNumPts[Index] > Count) ApScanNumPts[Index] = Count;
        %{pVar->Status = minibeamComputeOptimalPosition(
              pVar->ApOpdm[pVar->Index], pVar->ApScanPosData[pVar->Index],
              pVar->ApScanDetData[pVar->Index], 0,
              pVar->ApScanNumPts[pVar->Index], &pVar->OptimalPosition);}%
        if (Status == 0) {
          ApSugPreset[Index] = OptimalPosition;
          pvPut(ApSugPreset[Index]);
          ApComputeOptReturn[Index] = COMPUTE_OPT_RETURN_OK;
        } else {
          %{errlogPrintf("%s(P=%s,D=%s): Failed to determine optimal "
                    "position; minibeamComputeOptimalPosition returned %d\n",
                PROGRAM_NAME, pVar->MacroP, pVar->MacroD, pVar->Status);}%
        }
      } else {
        %{errlogPrintf("%s(P=%s,D=%s): Failed to read PV values in order "
                  "to determine optimal position\n",
              PROGRAM_NAME, pVar->MacroP, pVar->MacroD);}%
      }
      pvPut(ApComputeOptReturn[Index]);
      pvFlush();
      ApComputeOpt[Index] = COMPUTE_OPT_NO;
      pvPut(ApComputeOpt[Index]);
    } state waitForComputationNeeded
  }
}
