#!/bin/sh

appName='minibeam'

if [ "X$1" = 'X-h' -o "X$1" = 'X--help' ]; then
  echo 'Usage: control.sh [P [D [PM [MC [MH [MV]]]]]]'
  echo 'Parameters:'
  echo '  P   PV prefix (default: "ioc23:")'
  echo '  D   Mini-beam apparatus device name (default: "Mb1")'
  echo '  PM  Motor PV prefix (default: "ioc23:")'
  echo '  MC  Carrier stage motor (default: "m1")'
  echo '  MH  Aperture stage horizontal motor (default: "m2")'
  echo '  MV  Aperture stage vertical motor (default: "m3")'
  echo 'Example: control.sh'
  echo 'Example: control.sh ioc1: Mb2'
  exit 0
fi

pvPrefix='ioc23:'
if [ "X$1" != 'X' ]; then pvPrefix="$1"; fi
deviceName='Mb1'
if [ "X$2" != 'X' ]; then deviceName="$2"; fi
motorPvPrefix='ioc23:'
if [ "X$3" != 'X' ]; then motorPvPrefix="$3"; fi
motorC='m1'
if [ "X$4" != 'X' ]; then motorC="$4"; fi
motorH='m2'
if [ "X$5" != 'X' ]; then motorH="$5"; fi
motorV='m3'
if [ "X$6" != 'X' ]; then motorV="$6"; fi

thisScriptHome=`dirname "$0"`
cd "$thisScriptHome"
if [ $? -ne 0 ]; then
  echo "Error: failed to cd to \"$thisScriptHome\"." 1>&2
  exit 1
fi
thisScriptHome=`pwd`
if [ $? -ne 0 ]; then
  echo 'Error: failed to get current working directory.' 1>&2
  exit 1
fi

cd "../${appName}App/op/adl"
if [ $? -ne 0 ]; then
  echo -n 'Error: failed to cd to ' 1>&2
  echo "\"$thisScriptHome/../${appName}App/op/adl\"." 1>&2
  exit 1
fi

pMacro="$pvPrefix,D=$deviceName,PM=$motorPvPrefix,MC=$motorC,MH=$motorH"
pMacro="${pMacro},MV=$motorV"
exec medm -x -macro "P=$pMacro" ./minibeam.adl
